/*
**  $Id: DB_host.c,v 1.3 1999/04/27 21:57:39 jms Exp $
**
**  GXSNMP -- An snmp management application
**  Copyright (c) 1998 Gregory McLean
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc.,  59 Temple Place - Suite 330, Cambridge, MA 02139, USA.
*/

/*
**  host.c contains utility functions related to the creation and destruction
**  of DB_host items, and functions used to manage the DB_host database.
*/

#include <gnome.h>
#include "g_sqldb.h"
#include "tables.h"
#include "DB_host.h"
#include "db_interface.h"
#include "interface.h"
#include "graph.h"

static void DB_host_class_init      (DB_hostClass     * klass);
static void DB_host_init            (DB_host          * dbh);
static void DB_host_destroy	    (GtkObject        * object);

static GtkDatabaseItemClass * parent_class;

/******************************************************************************
**
**  DB_host_get_type ()
**
******************************************************************************/

GtkType
DB_host_get_type()
{
  static GtkType DB_host_type = 0;

  if (!DB_host_type)
    {
      GtkTypeInfo DB_host_info =
      {
        "DB_host",
        sizeof (DB_host),
        sizeof (DB_hostClass),
        (GtkClassInitFunc)  DB_host_class_init,
        (GtkObjectInitFunc) DB_host_init,
        /* reserved 1 */ NULL,
        /* reserved 2 */ NULL,
        (GtkClassInitFunc) NULL,
      };
      DB_host_type = gtk_type_unique (gtk_database_item_get_type (), 
				      &DB_host_info);
    }
  return DB_host_type;
}

/******************************************************************************
**
**  Class initialization subroutine for the DB_host class.
**
******************************************************************************/

static void
DB_host_class_init (DB_hostClass * class)
{
  GtkObjectClass *object_class;

  object_class = (GtkObjectClass *) class;

  parent_class = gtk_type_class (gtk_database_item_get_type ());

  object_class->destroy = DB_host_destroy;
}

/******************************************************************************
**
**  Object initialization subroutine for DB_host objects
**
******************************************************************************/

static void
DB_host_init (DB_host * dbh)
{
  dbh->created  = g_strdup (_("(not in database)"));
  dbh->modified = g_strdup (dbh->created);
}

/******************************************************************************
**
**  Subroutine to create a new DB_host object
**
******************************************************************************/

DB_host *
DB_host_new (void)
{
  return gtk_type_new (DB_host_get_type ());
}

/******************************************************************************
**
**  Function to destroy a DB_host object.  The rowid must be zero, which
**  marks it as being "not in the database"
**
******************************************************************************/

void
DB_host_destroy (GtkObject * object)
{
  DB_host * dbh;

  g_return_if_fail (object != NULL);
  g_return_if_fail (DB_IS_HOST (object));

  dbh = DB_HOST (object);

  g_return_if_fail (dbh->rowid == 0);

  if (dbh->created)     g_free (dbh->created);
  if (dbh->modified)    g_free (dbh->modified);
  if (dbh->dns_name)    g_free (dbh->dns_name);
  if (dbh->name)        g_free (dbh->name);
  if (dbh->description) g_free (dbh->description);
  if (dbh->contact)     g_free (dbh->contact);

  if (GTK_OBJECT_CLASS (parent_class)->destroy)
    (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
}

/******************************************************************************
**
**  Function to add a DB_host object to the database.  The rowid must be
**  zero, meaning that the item is not already in the database.
**
******************************************************************************/

void
host_add (DB_host * dbh)
{
  g_assert (dbh->rowid == 0);
  dbh->rowid = g_sqldb_highest_rowid (host_sqldb, "_rowid") + 1;

  if (dbh->created) g_free (dbh->created);
  dbh->created  = db_timestamp ();

  if (dbh->modified) g_free (dbh->modified);
  dbh->modified = g_strdup (dbh->created);

  g_sqldb_row_add (host_sqldb, dbh);
}

/******************************************************************************
**
**  Function to update a DB_host object in the database.  The rowid must be
**  non-zero, meaning that the item is already in the database.
**
******************************************************************************/

void
host_update (DB_host * dbh)
{
  g_assert (dbh->rowid != 0);

  if (dbh->modified) g_free (dbh->modified);
  dbh->modified = db_timestamp ();

  g_sqldb_row_update (host_sqldb, dbh);
}

/******************************************************************************
**
**  Function to delete a DB_host object from the database.  The rowid must
**  be non-zero, meaning that the item is already in the database.
**
**  When we delete a DB_host object from the database, we must also delete 
**  any DB_graph entries which describe this host on a map.  Interfaces
**  owned by the host are unassigned and become orphaned ... unassigned to
**  any host.
**
**  When we finished, we set the rowid to zero, meaning that the item is
**  no longer in the database, and return.  The caller is responsible for
**  making a call to host_destroy to free the memory associated with the
**  DB_host block.
**
******************************************************************************/

void
host_delete (DB_host * dbh)
{
  GList        * gl;
  DB_interface * dbi;

  g_assert (dbh->rowid != 0);
  
  while ((gl = dbh->DB_interfaces))	/* Assignment intended */
    {
      dbi = (DB_interface *) gl->data;
      g_assert (dbi->host == dbh->rowid);
      dbi->host = 0;			    
      interface_update (dbi);
    }

  while ((gl = dbh->DB_graphs))		/* Assignment intended */
    {
      graph_delete  ((DB_graph *) gl->data);
      graph_destroy ((DB_graph *) gl->data);
    }

  g_assert (g_sqldb_row_delete (host_sqldb, dbh) == TRUE); 

  dbh->rowid = 0;
}

/******************************************************************************
**
**  Function to return the master host GList
**
******************************************************************************/

GList *
host_list (void)
{
  return g_sqldb_table_list (host_sqldb);
}


/******************************************************************************
**
**  Function to locate a host by the database rowid
**
******************************************************************************/

DB_host *
host_find_by_rowid (guint rowid)
{
  return g_sqldb_row_find (host_sqldb, "_rowid", &rowid);
}

/******************************************************************************
**
**  Function to locate a host by map name
**
**  Since g_sqldb_row_find is broken for string keys, just do a quick search
**  for now.
**
******************************************************************************/

DB_host *
host_find_by_name (gchar * name)
{
  GList * gl;

  gl = g_sqldb_table_list (host_sqldb);
  while (gl)
    {
      DB_host * dbh;
 
      dbh = (DB_host *) gl->data;
      if (!strcmp (dbh->name, name))
	return dbh;
      gl = gl->next;
    }
  return NULL;
}

/* EOF */

